/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Eina.h>
#include <media_content.h>
#include "dbg.h"
#include "common.h"
#include "i18n.h"

#include "AppCommon.h"
#include "FileList.h"
#include "ExtNameInfo.h"
#include "FileInfo.h"
#include "DirectoryInfo.h"
#include "Mediadata.h"
#include "MediadataImpl.h"

struct SMediadataImpl {
	Eina_List *filelist;

	filter_h filter;
	int source_type;
	int media_type;
};

bool CMediadataImpl::m_CreateFilter(int source_type, int media_type)
{
	char buf[1024];
	int ret;

	struct SPairSources {
		sources source;
		const char* string;
	} source_condn[] = {
		{ E_ALL, "" },
		{ E_TV, "(MEDIA_STORAGE_TYPE=0)" },
		{ E_USB, "(MEDIA_STORAGE_TYPE=1)" }
	};
	ASSERT(source_condn[E_ALL].source == E_ALL &&
		source_condn[E_TV].source == E_TV &&
		source_condn[E_USB].source == E_USB);

	struct SPairMedia {
		enum groups groups;
		const char* string;
	} media_condn[] = {
		{ E_GRP_ALL, "" },
		{ E_GRP_PHOTO, "(MEDIA_TYPE=0)" },
		{ E_GRP_VIDEO, "(MEDIA_TYPE=1)" },
		{ E_GRP_MUSIC, "(MEDIA_TYPE=3)" }
	};
	ASSERT(media_condn[E_GRP_ALL].groups == E_GRP_ALL &&
		media_condn[E_GRP_PHOTO].groups == E_GRP_PHOTO &&
		media_condn[E_GRP_VIDEO].groups == E_GRP_VIDEO &&
		media_condn[E_GRP_MUSIC].groups == E_GRP_MUSIC);

	ret = media_filter_create(&m->filter);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("Media Filter Creation Failed");
		return false;
	}

	_DBG("media: %d, source: %d", media_type, source_type);
	if (media_type == E_GRP_ALL) {
		if (source_type == E_ALL)
			return true;

		snprintf(buf, sizeof(buf), "%s", source_condn[source_type].string);
	}
	else {
		if (source_type == E_ALL)
			snprintf(buf, sizeof(buf), "%s", media_condn[media_type].string);
		else
			snprintf(buf, sizeof(buf), "%s AND %s", source_condn[source_type].string, media_condn[media_type].string);
	}

	media_filter_set_condition(m->filter, buf, MEDIA_CONTENT_COLLATE_RTRIM);

	return true;
}

void CMediadataImpl::m_DestroyFilter(filter_h filter)
{
	if (!filter)
		return;

	int r = media_filter_destroy(filter);
	ASSERT(r == MEDIA_CONTENT_ERROR_NONE);
}

bool CMediadataImpl::sm_CbMediaInfo(media_info_h media, void *dt)
{
	CMediadataImpl* root = (CMediadataImpl*)dt;
	
	bool ret = false;
	if (root)
		ret = root->m_OnMediaInfo(media);

	return ret;
}

bool CMediadataImpl::m_OnMediaInfo(media_info_h media)
{
	CFileInfo *pInfo = NULL;

	_CREATE_BEGIN{
		_CHECK(pInfo = new CFileInfo)
		_CHECK(pInfo->Create())
		_CHECK(pInfo->SetLoadInfo(media))
		_CHECK(pInfo->Load())
		_WHEN_SUCCESS{
			m->filelist = eina_list_append(m->filelist, pInfo);
		}
		_CHECK_FAIL{}
		_CHECK_FAIL{}
		_CHECK_FAIL{ pInfo->Destroy(); }
		_CHECK_FAIL{ delete pInfo; pInfo = NULL; }
	}_CREATE_END_AND_CATCH{ return false; }

	return true;
}

bool CMediadataImpl::sm_CbFolderInfo(media_folder_h folder, void* data)
{
	CMediadataImpl* root = (CMediadataImpl*)data;

	_DBG("root: %x, m: %x", root, root->m);
	bool ret = false;
	if (root)
		ret = root->m_OnFolderInfo(folder);

	return ret;
}

bool CMediadataImpl::m_OnFolderInfo(media_folder_h folder)
{
	CDirectoryInfo *pDir = NULL;

	_CREATE_BEGIN{
		_CHECK(pDir = new CDirectoryInfo)
		_CHECK(pDir->Create())
		_CHECK(pDir->SetLoadInfo(m->filter, folder))
		_CHECK(pDir->Load())
		_WHEN_SUCCESS{
			Add(m->media_type, pDir);
		}
		_CHECK_FAIL{}
		_CHECK_FAIL{}
		_CHECK_FAIL{ pDir->Destroy(); }
		_CHECK_FAIL{ delete pDir; pDir = NULL; }
	}_CREATE_END_AND_CATCH{ return false; }
	
	return true;
}

bool CMediadataImpl::m_GetMediaList(const char *folderid)
{
	t_Remove(&m->filelist);

	int r = media_folder_foreach_media_from_db(folderid, m->filter, sm_CbMediaInfo, this);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("Media folder foreach folder from db failed");
	}
	return r == MEDIA_CONTENT_ERROR_NONE ? true : false;
}

void CMediadataImpl::m_GetFolderList(void)
{
	int r;
	_DBG("this: 0x%x", this);
	r = media_folder_foreach_folder_from_db(m->filter, sm_CbFolderInfo, this);
	_DBG("");
	if (r != MEDIA_CONTENT_ERROR_NONE)
		_ERR("Media folder foreach folder from db failed");
}

void CMediadataImpl::m_UpdateFolderLists(void)
{
	int type;

	RemoveAll();

	for (type = E_GRP_ALL; type <= E_GRP_MUSIC; type++) {
		m->media_type = type;

		bool r = m_CreateFilter(m->source_type, m->media_type);
		if (!r) {
			_ERR("Media Filter Creation Failed");
			continue;
		}

		m_GetFolderList();

		m_DestroyFilter(m->filter);
		m->filter = NULL;
	}
}

bool CMediadataImpl::Create(void)
{
	if (m)
		return false;

	_CREATE_BEGIN{
		_CHECK(m = new SMediadataImpl)
		_CHECK(media_content_connect() == MEDIA_CONTENT_ERROR_NONE)
		_WHEN_SUCCESS{

		}
		_CHECK_FAIL{ media_content_disconnect(); }
		_CHECK_FAIL{ delete m; m = NULL; }
	} _CREATE_END_AND_CATCH{ return false; }
	
	m->source_type = E_GRP_NONE;
	m->media_type  = E_GRP_ALL;
	m->filelist = NULL;

	return true;
}

void CMediadataImpl::Destroy(void)
{
	if (!m)
		return;

	_DBG("#########################");

	t_Remove(&m->filelist);
	RemoveAll();

	int r;
	r = media_content_disconnect();
	if (r != MEDIA_CONTENT_ERROR_NONE)
		_ERR("Media Content Connect Failed");

	delete m;
	m = NULL;
}

bool CMediadataImpl::GetMediaList(const char* folderid, int media_type, int source_type, Eina_List **list)
{
	ASSERT(0 <= media_type && media_type <= t_Max());

	if (!m)
		return false;

	/* Inner Level */
	m->media_type = media_type;
	m->source_type = source_type;

	bool r = m_CreateFilter(m->source_type, m->media_type);
	if (!r) {
		_ERR("Media Filter Creation Failed");
		return false;
	}

	// Ư   ִ ϵ ̾ m->filelist .
	r = m_GetMediaList(folderid);

	m_DestroyFilter(m->filter);
	m->filter = NULL;

	if (m->filelist) {
		*list = eina_list_clone(m->filelist);
		r = true;
	}

	return r;
}

bool CMediadataImpl::GetFolderList(int media_type, int source_type, Eina_List **list)
{
	ASSERT(0 <= media_type && media_type <= t_Max());

	if (!m)
		return false;
		
	_DBG("sourceType: %d, m->source_Type: %d", source_type, m->source_type);
	/* Outer level */
	if (source_type != m->source_type) {
		m->source_type = source_type;
		m_UpdateFolderLists();
	}

	*list = Clone(media_type);
	return true;
}
