/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "dbg.h"
#include "i18n.h"
#include "define.h" 
#include "common.h"

#include <AppCommon.h>
#include "Mediadata.h"
#include "ExtNameInfo.h"
#include "SortName.h"
#include "GengridItemClass.h"
#include "InputHandler.h"
#include "FileGrid.h"
#include "Info.h"
#include <BaseView.h>
#include <InputHandler.h>
#include <ViewMgr.h>
#include "FbContextView.h"


#define GENGRID_ITEM_SIZE_W (263 + 20)
#define GENGRID_ITEM_SIZE_H (359 + 20)


static Elm_Object_Item *_find_item_by_id(Eina_List *list, char *fid)
{
	Eina_List *l;
	SItemInfo *pItemInfo;
	void* obj;
	char *id;

	EINA_LIST_FOREACH(list, l, obj) {
		pItemInfo = (SItemInfo *)obj;
		id = pItemInfo->pInfo->Id();
		if (id && !strcmp(id, fid))
			return pItemInfo->item;
	}

	return NULL;
}

static SItemInfo *_find_item_info(Eina_List *list, Elm_Object_Item *item)
{
	Eina_List *l;
	SItemInfo *pItemInfo;
	void* obj;

	EINA_LIST_FOREACH(list, l, obj) {
		pItemInfo = (SItemInfo*)obj;
		if (pItemInfo->item == item)
			return pItemInfo;
	}

	return NULL;
}


struct SFileGrid {
	CMediadata *pMediadata;

	Evas_Object *eoWin;
	Evas_Object *eoBase;
	Evas_Object *eoGrid;

	Eina_List   *elFile;
	Eina_List   *elItemInfo;

	CFileGrid::SCallback callback;

	int depth;
	SItemInfo *ctxt_itinfo;
	SItemInfo *handle_itinfo;

	char *fid;

	CGengridItemClass *pGengridItemClass;

	SFileGrid() {
		memset(this, 0, sizeof(SFileGrid));
	}
};


void CFileGrid::sm_CbRealized(void *data, Evas_Object *obj, void *event_info)
{
	_DBG();
	if (!data || !obj || !event_info)
		return;

	elm_object_item_signal_callback_add((Elm_Object_Item*)event_info,
		FBR_SIGNAL_BTN_CLICKED, FBR_BASE_VIEW,
		sm_CbSelectItem, data);
}

void CFileGrid::sm_CbUnrealized(void *data, Evas_Object *obj, void *event_info)
{
	_DBG();
	if (!data || !obj || !event_info)
		return;

	elm_object_item_signal_callback_del((Elm_Object_Item*)event_info,
		FBR_SIGNAL_BTN_CLICKED, FBR_BASE_VIEW,
		sm_CbSelectItem);
}

void CFileGrid::Action(void)
{
	m->handle_itinfo = m->ctxt_itinfo;
	m_HandleSelection();
	m_DeleteContextView();
}

void CFileGrid::m_DeleteContextView(void)
{
	if(CViewMgr::GetInstance()->CheckTop(FBR_CONTEXT_VIEW))
		CViewMgr::GetInstance()->PopView();
}

void CFileGrid::sm_CbSelectItem(void *data, Elm_Object_Item *it, const char *emission, const char *source)
{
	CFileGrid* root = (CFileGrid*)data;
	SItemInfo *pItemInfo;

	if (!data || !it)
		return;

	pItemInfo = _find_item_info(root->m->elItemInfo, it);
	if (!pItemInfo) {
		_ERR(" no item info found ");
		return;
	}

	root->m->handle_itinfo = pItemInfo;
	root->m_HandleSelection();
}

void CFileGrid::m_HandleSelection(void)
{
	char *path;
	int type;
	char *id;
	SItemInfo *pItemInfo = m->handle_itinfo;

	if (!pItemInfo)
		return;

	type = pItemInfo->pInfo->Type();
	if (type == E_GRP_FOLDER) {
		if (m->fid) {
			free(m->fid);
			m->fid = NULL;
		}
		id = pItemInfo->pInfo->Id();
		if (!id)
			return;

		m->fid = strdup(id);
		if (!m->fid)
			return;

		elm_object_tree_focus_allow_set(m->eoBase, EINA_FALSE);
		t_Update(m->fid, false);//_update_filegrid(m, m->fid, false);
		elm_object_tree_focus_allow_set(m->eoBase, EINA_TRUE);
		elm_object_focus_set(m->eoGrid, EINA_TRUE);
	}
	else {
		path = pItemInfo->pInfo->Path();
		if (!path)
			return;

		if (m->callback.cbLinkApp)
			m->callback.cbLinkApp(m->callback.cookie, type, CInfo::SourceType(), path);
		elm_gengrid_item_selected_set(pItemInfo->item, EINA_FALSE);
	}
}

void CFileGrid::m_GetMediaList(char *fid)
{
	bool ret;
	ret = m->pMediadata->GetMediaList(fid, CInfo::GroupIndex(), CInfo::SourceType(), &m->elFile);
	if (ret == false) {
		_ERR(" Fetching all list from file failed ");
		return;
	}
}

void CFileGrid::m_GetFolderList(void)
{
	bool ret;
	ret = m->pMediadata->GetFolderList(CInfo::GroupIndex(), CInfo::SourceType(), &m->elFile);
	if (ret == false) {
		_ERR(" Fetching all list from file failed ");
		return;
	}
}

void CFileGrid::t_CreateFileGrid(void)
{
	Evas_Object *grid;
	
	grid = elm_gengrid_add(m->eoBase);
	if (!grid)
		return;
	evas_object_size_hint_weight_set(grid,
		EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_gengrid_align_set(grid, 0, 0);
	elm_object_part_content_set(m->eoBase, FBR_PART_MEDIA_GRID, grid);


	Connect(grid);

	evas_object_smart_callback_add(grid, FBR_SIGNAL_REALIZED,      sm_CbRealized, this);
	evas_object_smart_callback_add(grid, FBR_SIGNAL_UNREALIZED,    sm_CbUnrealized, this);
	elm_gengrid_item_size_set(grid,
		elm_config_scale_get() * GENGRID_ITEM_SIZE_W,
		elm_config_scale_get() * GENGRID_ITEM_SIZE_H);

	m->eoGrid = grid;

	m->pGengridItemClass = new CGengridItemClass;
	if (!m->pGengridItemClass->Create()) {
		delete m->pGengridItemClass;
		m->pGengridItemClass = NULL;
		evas_object_del(grid);
		_ERR(" elm_genlist_item_class_new failed ");
		return;
	}
	
	evas_object_show(grid);
}


void CFileGrid::t_Empty(bool sort_flag)
{
	if (m->eoGrid) {
		elm_gengrid_clear(m->eoGrid);
		evas_object_del(m->eoGrid);
		m->eoGrid = NULL;
	}

	if (m->pGengridItemClass)
	{
		m->pGengridItemClass->Destroy();
		delete m->pGengridItemClass;
		m->pGengridItemClass = NULL;
	}

	if (sort_flag == false) {
		eina_list_free(m->elFile);
		m->elFile = NULL;
	}

	m->elItemInfo = NULL;
	if (m->callback.cbUpdateEmptyFocusSequence)
		m->callback.cbUpdateEmptyFocusSequence(m->callback.cookie);
}


void CFileGrid::t_Update(char *fid, bool sort_flag)
{
	_DBG("fid: %s, sort_flag: %d", fid, sort_flag);
	t_Empty(sort_flag);
	t_CreateFileGrid();

	if (sort_flag == false) {
		if (fid)
			m_GetMediaList(fid);
		else
			m_GetFolderList();
	}
	m->elFile = CSortName::Sort(m->elFile, (CSortName::ESort)CInfo::SortType());
	t_Fill();

	if (fid)    // folder
		m->depth = 1;
	else
		m->depth = 0;
}


void CFileGrid::t_Fill(void)
{
	CExtNameInfo *pInfo;
	Eina_List *l;
	SItemInfo *pItemInfo;
	void* obj;
	int ts;

	_DBG("grid: %x, flist: %x", m->eoGrid, m->elFile);
	if (!m->eoGrid || !m->elFile)
		return;

	ts = eina_list_count(m->elFile);
	_DBG("count: %d", ts);
	if (ts == 0) {
		t_Empty(false);
		return;
	}

	EINA_LIST_FOREACH(m->elFile, l, obj) {
		pInfo = (CExtNameInfo*)obj;
		pItemInfo = (SItemInfo *)calloc(1, sizeof(SItemInfo));
		if (!pItemInfo) {
			t_Empty(false);
			return;
		}

		pItemInfo->pInfo = pInfo;
		pItemInfo->item = elm_gengrid_item_append(m->eoGrid, m->pGengridItemClass->Handle(), pItemInfo, NULL, m);
		m->elItemInfo = eina_list_append(m->elItemInfo, pItemInfo);
	}

	if (m->callback.cbUpdatedGrid)
		m->callback.cbUpdatedGrid(m->callback.cookie);
}


bool CFileGrid::Create(Evas_Object* win, Evas_Object *base, const SCallback *cb)
{
	ASSERT(!m);
	ASSERT(cb);

	_CREATE_BEGIN{
		_CHECK(m = new SFileGrid)
		_CHECK(m->pMediadata = new CMediadata )
		_CHECK(m->pMediadata->Create())

		_WHEN_SUCCESS{}

		_CHECK_FAIL{ m->pMediadata->Destroy(); }
		_CHECK_FAIL{ delete m->pMediadata; }
		_CHECK_FAIL{ delete m; m = NULL; }
	} _CREATE_END_AND_CATCH{ return false; }

	m->eoWin    = win;
	m->eoBase   = base;
	m->callback = *cb;

	return true;
}


void CFileGrid::Destroy(void)
{
	ASSERT(m);

	t_Empty(false);

	m_DeleteContextView();

	free(m->fid);

	m->pMediadata->Destroy();
	delete m->pMediadata;
	delete m;
	m = NULL;
}


void CFileGrid::Update(bool sort_flag)
{
	ASSERT(m);

	char* fid = NULL;
	if (sort_flag == true) {
		fid = m->fid;
	}
	t_Update(fid, sort_flag);
}


void CFileGrid::SetFocus(Eina_Bool flag)
{
	ASSERT(m);

	elm_object_focus_set(m->eoGrid, flag);
}


Evas_Object* CFileGrid::Grid(void)
{
	ASSERT(m);

	return m->eoGrid;
}


void CFileGrid::OnKeyUp(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Up *ev) {
	_DBG();
	Elm_Object_Item *it;
	SItemInfo *pItemInfo;

	if (strcmp(ev->keyname, KEY_BACK) == 0) {
		if (m->depth == 0) {
			if (m->callback.cbUnsetFocus)
				m->callback.cbUnsetFocus(m->callback.cookie);
		}
		else {
			t_Update(NULL, false);
			it = _find_item_by_id(m->elItemInfo, m->fid);
			elm_gengrid_item_show(it, ELM_GENGRID_ITEM_SCROLLTO_IN);
			elm_object_item_focus_set(it, EINA_TRUE);
		}
	}
	else if (strcmp(ev->keyname, KEY_MENU) == 0) {
		it = elm_object_focused_item_get(obj);
		if (!it) {
			_ERR(" unable to get focused item ");
			return;
		}
		pItemInfo = _find_item_info(m->elItemInfo, it);
		if (!pItemInfo)
			return;

		m->ctxt_itinfo = pItemInfo;

		CViewMgr::GetInstance()->PushView(FBR_CONTEXT_VIEW, pItemInfo->pInfo);
	}
}


void CFileGrid::OnMouseMove(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev) {
	_DBG();
	Elm_Object_Item *item;

	if (!obj)
		return;

	item = elm_gengrid_at_xy_item_get(obj, ev->cur.canvas.x,
		ev->cur.canvas.y, NULL, NULL);
	if (!item)
		return;

	elm_object_item_focus_set(item, EINA_TRUE);
}
