/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "i18n.h"
#include "define.h"
#include "common.h"
#include "dbg.h"
#include <media_content.h>
#include "ExtNameInfo.h"
#include "DirectoryInfo.h"
#include "GengridItemClass.h"

const char *typestr[] = {
	N_(""),
	N_("Photo"),
	N_("Video"),
	N_("Music"),
	N_("Other"),
	N_("Folder"),
};

void CGengridItemClass::sm_SetTypeimage(Evas_Object *img, int type)
{
	char buf[MAX_LENGTH];

	if (!img)
		return;

	switch (type) {
	case E_GRP_VIDEO:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_NORMAL_VIDEO);
		break;
	case E_GRP_MUSIC:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_NORMAL_MUSIC);
		break;
	default:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_NORMAL_OTHER);
		break;
	}

	elm_image_file_set(img, buf, NULL);
}

Evas_Object *CGengridItemClass::sm_GetIcon(Evas_Object *obj, CExtNameInfo *pInfo)
{
	Evas_Object *icon;
	char *path;

	path = pInfo->Path();
	if (!path)
		return NULL;

	icon = elm_icon_add(obj);
	if (!icon)
		return NULL;

	elm_need_ethumb();
	elm_icon_thumb_set(icon, path, NULL);
	elm_image_aspect_fixed_set(icon, EINA_FALSE);

	return icon;
}

Evas_Object *CGengridItemClass::sm_AddThumbnail(Evas_Object *obj, CExtNameInfo *pInfo)
{
	char *path;
	char buf[MAX_LENGTH];
	int type;
	Evas_Object *img;

	if (!obj || !pInfo)
		return NULL;

	type = pInfo->Type();
	path = pInfo->ThumbnailPath();
	_DBG("path: %x", path);

	if (!path && type == E_GRP_PHOTO) {
		img = sm_GetIcon(obj, pInfo);
		return img;
	}

	img = elm_image_add(obj);
	if (!img)
		return NULL;
	evas_object_show(img);

	if (path) {
		elm_image_file_set(img, path, "NULL");
		elm_image_aspect_fixed_set(img, EINA_FALSE);
		return img;
	}

	switch (type) {
	case E_GRP_FOLDER:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_FOLDER);
		break;
	case E_GRP_VIDEO:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_DEFAULT_VIDEO);
		break;
	case E_GRP_MUSIC:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_DEFAULT_MUSIC);
		break;
	case E_GRP_OTHER:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			FBR_IMAGE_DEFAULT_OTHER);
		break;
	}
	elm_image_no_scale_set(img, EINA_TRUE);
	elm_image_aspect_fixed_set(img, EINA_TRUE);
	elm_image_file_set(img, buf, NULL);
	return img;
}


char *CGengridItemClass::sm_CbTextGet(void *data, Evas_Object *obj, const char *part)
{
	if (!data || !obj)
		return NULL;

	SItemInfo *itinfo = (SItemInfo *)data;
	CExtNameInfo *pInfo  = itinfo->pInfo;

	char *txt;
	char buf[MAX_LENGTH];
	int count;
	int type;

	if (!strcmp(part, "elm.text")) {
		txt = pInfo->Name();
		if (txt)
			return strdup(txt);
	}
	else if (!strcmp(part, "elm.text1")) {
		type = pInfo->Type();
		if (type == E_GRP_FOLDER) {
			CDirectoryInfo* pDirInfo = (CDirectoryInfo*)pInfo;
			count = pDirInfo->Count();
			snprintf(buf, sizeof(buf), "%d", count);
			return strdup(buf);
		}
		else
			return strdup(typestr[type]);
	}

	return NULL;
}

Evas_Object *CGengridItemClass::sm_CbContentGet(void *data, Evas_Object *obj, const char *part)
{
	CExtNameInfo *pInfo;
	SItemInfo *itinfo;
	Evas_Object *img;
	int type;
	char *path;

	if (!data || !obj)
		return NULL;

	itinfo = (SItemInfo *)data;
	pInfo = itinfo->pInfo;

	if (!strcmp(part, "elm.swallow.icon")) {
		img = sm_AddThumbnail(obj, pInfo);
		return img;
	}
	else if (!strcmp(part, "elm.swallow.icon1")) {
		type = pInfo->Type();
		if (type == E_GRP_FOLDER || type == E_GRP_PHOTO)
			return NULL;

		path = pInfo->ThumbnailPath();
		if (!path)
			return NULL;

		img = elm_image_add(obj);
		if (!img)
			return NULL;

		sm_SetTypeimage(img, type);
		elm_image_no_scale_set(img, EINA_TRUE);
		evas_object_show(img);
		return img;
	}

	return NULL;
}

void CGengridItemClass::sm_CbRemove(void *data, Evas_Object *obj)
{
	free(data);
}

bool CGengridItemClass::Create(void)
{
	if (m_pItemClass)
		return false;

	m_pItemClass = elm_gengrid_item_class_new();
	if (!m_pItemClass) {
		return false;
	}

	m_pItemClass->item_style = FBR_STYLE_MEDIA_GRID;
	m_pItemClass->func.text_get    = sm_CbTextGet;
	m_pItemClass->func.content_get = sm_CbContentGet;
	m_pItemClass->func.state_get   = NULL;
	m_pItemClass->func.del         = sm_CbRemove;

	return true;
}

void CGengridItemClass::Destroy(void)
{
	if (!m_pItemClass)
		return;

	elm_gengrid_item_class_free(m_pItemClass);
	m_pItemClass = NULL;
}

Elm_Gengrid_Item_Class* CGengridItemClass::Handle(void)
{
	return m_pItemClass;
}
