/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <utilX.h>
#include "i18n.h"
#include "dbg.h"

#include "AppCommon.h"
#include "InputHandler.h"
#include "define.h"
#include "common.h"
#include "ExtNameInfo.h"
#include <BaseView.h>
#include <ViewMgr.h>
#include "FbContextView.h"
#include "Dummy/DummyContent.h"

//#define _GET_PRIV(o) evas_object_data_get(o, "CTMDATA");
//#define _SET_PRIV(o, data) evas_object_data_set(o, "CTMDATA", data);

#define FBR_TEXT_SIZE N_("Size")
#define FBR_TEXT_FORMAT N_("Format")
#define FBR_TEXT_OPEN N_("OPEN")
#define FBR_TEXT_LINK N_("Other linkable apps")
#define FBR_TEXT_REL N_("Other apps for this type")
#define EMPTY_STR ""
#define TOTAL_CONTEXT_BTNS 5
#define TABLE_MAX_COL 2
#define MIN_LABEL_WIDTH 103
#define FBR_CTXT_BTN_WPAD 26
#define FBR_CTXT_TABLE_WPAD 14
#define FBR_CTXT_TABLE_HPAD 14


struct SContextView {
	Evas_Object *eoWin;
	Evas_Object *eoBase;
	Evas_Object *eoBtnOpen;
	Evas_Object *rel_first[TABLE_MAX_COL];
	Evas_Object *rel_last[TABLE_MAX_COL];
	Evas_Object *link_first[TABLE_MAX_COL];
	Evas_Object *link_last[TABLE_MAX_COL];

	CExtNameInfo *finfo;

	CDummyContent content;

	SContextView() {
		eoWin = NULL;
		eoBase = NULL;
		eoBtnOpen = NULL;
		int a;
		for (a = 0; a < TABLE_MAX_COL; a++) {
			rel_first[a] = NULL;
			rel_last[a] = NULL;
			link_first[a] = NULL;
			link_last[a] = NULL;
		}
		finfo = NULL;
		pHandlerButton = new SContextView::CHandlerButton(this);
	}
	~SContextView() {
		delete pHandlerButton;
	}

	class CHandlerButton : public CListenerMgr, IMouseMoveListener, IMouseUpListener, IKeyUpListener {
		SContextView *m;
	public:
		CHandlerButton(SContextView *ins) : IMouseMoveListener(this), IMouseUpListener(this), IKeyUpListener(this) { m = ins; }

		virtual void OnMouseMove(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev) {
			if (!elm_object_focus_get(obj))
				elm_object_focus_set(obj, EINA_TRUE);
		}
		virtual void OnKeyUp(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Up *ev) {
			_DBG();
			if (m->eoBtnOpen == obj) {
				_DBG();
				if (strcmp(ev->keyname, KEY_ENTER))
					return;

				CViewMgr::GetInstance()->UpdateView(FBR_BASE_VIEW, NULL);
			}
			else {
				_DBG();
				if (strcmp(ev->keyname, KEY_ENTER))
					return;

				elm_object_signal_emit(obj, FBR_SIGNAL_CONTENT_CLICKED, FBR_CONTEXT_VIEW);
			}
		}
		virtual void OnMouseUp(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Up *ev) {
			_DBG();
			if (m->eoBtnOpen == obj) {
				CViewMgr::GetInstance()->UpdateView(FBR_BASE_VIEW, NULL);
			}
			else {
				elm_object_signal_emit(obj, FBR_SIGNAL_CONTENT_CLICKED, FBR_CONTEXT_VIEW);
			}
		}
	} *pHandlerButton;
};


extern const char *typestr[];


void CContextView::_lbl_start_slide(Evas_Object *lbl)
{
	elm_label_ellipsis_set(lbl, EINA_FALSE);
	elm_label_slide_mode_set(lbl, ELM_LABEL_SLIDE_MODE_AUTO);
	elm_label_slide_go(lbl);
}

void CContextView::_btn_focused(void *data, Evas_Object *obj, void *ev)
{
	_DBG();
	if (!obj)
		return;

	elm_object_signal_emit(obj, FBR_SIGNAL_FOCUSED, FBR_CONTEXT_VIEW);

}

void CContextView::_btn_unfocused(void *data, Evas_Object *obj, void *ev)
{
	_DBG();
	if (!obj)
		return;

	elm_object_signal_emit(obj, FBR_SIGNAL_UNFOCUSED, FBR_CONTEXT_VIEW);
}


bool CContextView::_add_btn_label(Evas_Object *btn, const char *txt)
{
	Evas_Object *lbl;
	char buf[MAX_LENGTH];

	if (!btn || !txt)
		return false;

	lbl = elm_label_add(btn);
	if (!lbl) {
		_ERR("elm_label_add failed.");
		return false;
	}

	snprintf(buf, sizeof(buf),
			"<align=center><color=#FFFFFFFF>"
			"<font=%s>"
			"<font_size=21>%s</font_size>"
			"</font></color></align>", FONT_STYLE_MEDIUM, _(txt));
	elm_object_text_set(lbl, buf);
	elm_label_line_wrap_set(lbl, ELM_WRAP_WORD);
	evas_object_size_hint_min_set(lbl, MIN_LABEL_WIDTH, 0);
	elm_object_part_content_set(btn, FBR_PART_CTXT_BTNTEXT, lbl);

	return true;
}

bool CContextView::_add_btn_icon(Evas_Object *btn, const char *icon)
{
	Evas_Object *img;
	char buf[MAX_LENGTH];

	if (!btn || !icon)
		return false;

	img = elm_image_add(btn);
	if (!img) {
		_ERR("elm_image_add failed.");
		return false;
	}

	snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR, icon);
	elm_image_file_set(img, buf, NULL);
	elm_object_part_content_set(btn, FBR_PART_CTXT_BTNICON, img);

	return true;
}

bool CContextView::_add_btn_info(Evas_Object *btn, const char *icon, const char *txt)
{
	if (!btn || !icon || !txt)
		return false;

	if (!_add_btn_icon(btn, icon))
		return false;

	if (!_add_btn_label(btn, txt))
		return false;

	return true;
}


Evas_Object *CContextView::_add_btnbox(Evas_Object *eoBase)
{
	Evas_Object *box;

	if (!eoBase)
		return NULL;

	box = elm_box_add(eoBase);
	if (!box) {
		_ERR(" elm_box_add failed ");
		return NULL;
	}

	elm_box_horizontal_set(box, EINA_TRUE);
	elm_box_padding_set(box, FBR_CTXT_BTN_WPAD, 0);
	elm_box_align_set(box, 0, 0.5);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);

	return box;
}

bool CContextView::_get_context_details(CExtNameInfo *pInfo, char **path, char **title, char *subtitle, char *subtitle2)
{
	char buf[MAX_LENGTH];
	TSize size;
	double size_mb;
	int type;

	if (!pInfo)
		return false;

	*title = pInfo->Name();
	*path  = pInfo->ThumbnailPath();
	size = pInfo->Size();

	size_mb = (double)size / (1024 * 1024); /* Convert to MB */

	snprintf(buf, sizeof(buf), "%s : %f MB", _(FBR_TEXT_SIZE), size_mb);
	strcpy(subtitle, buf);

	type = pInfo->Type();
	snprintf(buf, sizeof(buf), "%s : %s", _(FBR_TEXT_FORMAT), _(typestr[type]));
	strcpy(subtitle2, buf);

	return true;
}

Evas_Object *CContextView::_add_rltv_ctnt(Evas_Object *obj, const char *name, const char *icon)
{
	Evas_Object *ly, *img;
	char buf[MAX_LENGTH];

	if (!obj) {
		_ERR("Invalid argument.");
		return NULL;
	}

	ly = elm_layout_add(obj);
	if (!ly) {
		_ERR("elm_layout_add failed.");
		return NULL;
	}

	elm_layout_file_set(ly, EDJEFILE, FBR_CONTEXT_APP);
	evas_object_size_hint_weight_set(ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	img = elm_image_add(ly);
	if (!img) {
		_ERR("elm_image_add failed.");
		evas_object_del(ly);
		return NULL;
	}

	if (!icon || !strcmp(icon, EMPTY_STR)) {
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR, FBR_IMAGE_DEFAULT_APP);
		elm_image_file_set(img, buf, NULL);
		elm_image_no_scale_set(img, EINA_TRUE);
	} else {
		elm_image_file_set(img, icon, NULL);
		elm_image_aspect_fixed_set(img, EINA_FALSE);
	}

	elm_object_part_content_set(ly, FBR_PART_CTXT_RELCTNT_ICON, img);
	elm_object_part_text_set(ly, FBR_PART_CTXT_RELCTNT_NAME, name);
	elm_object_focus_allow_set(ly, EINA_TRUE);

	return ly;
}


Evas_Object *CContextView::_add_scroller(Evas_Object *eoBase)
{
	Evas_Object *scroller;

	if (!eoBase) {
		_ERR("Invalid argument.");
		return NULL;
	}

	scroller = elm_scroller_add(eoBase);
	if (!scroller) {
		_ERR("elm_scroller_add failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL,
			EVAS_HINT_FILL);

	return scroller;
}

Evas_Object *CContextView::_add_table(Evas_Object *eoBase)
{
	Evas_Object *table;

	if (!eoBase) {
		_ERR("Invalid argument.");
		return NULL;
	}

	table = elm_table_add(eoBase);
	if (!table) {
		_ERR("elm_table_add failed.");
		return NULL;
	}

	elm_table_homogeneous_set(table, EINA_TRUE);
	elm_table_padding_set(table,
			FBR_CTXT_TABLE_WPAD * elm_config_scale_get(),
			FBR_CTXT_TABLE_HPAD * elm_config_scale_get());
	evas_object_size_hint_align_set(table, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(table);

	return table;
}


Evas_Object *CContextView::_get_icon(Evas_Object *obj, CExtNameInfo *finfo)
{
	Evas_Object *icon;
	char *path;

	path = finfo->Path();
	if (!path)
		return NULL;

	icon = elm_icon_add(obj);
	if (!icon)
		return NULL;

	elm_need_ethumb();
	elm_icon_thumb_set(icon, path, NULL);
	elm_image_aspect_fixed_set(icon, EINA_FALSE);

	return icon;
}


void CContextView::_set_default_thumb(Evas_Object *img, int type)
{
	char buf[MAX_LENGTH];

	if (!img)
		return;

	switch (type) {
	case E_GRP_FOLDER:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
				FBR_IMAGE_FOLDER);
		break;

	case E_GRP_VIDEO:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
				FBR_IMAGE_DEFAULT_VIDEO);
		break;

	case E_GRP_MUSIC:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
				FBR_IMAGE_DEFAULT_MUSIC);
		break;

	default:
		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
				FBR_IMAGE_DEFAULT_OTHER);
		break;
	}

	elm_image_file_set(img, buf, NULL);
	elm_image_no_scale_set(img, EINA_TRUE);
}


Evas_Object *CContextView::_create_lbl(Evas_Object *ly, char *title)
{
	Evas_Object *lbl;
	const char *s;
	char buf[MAX_LENGTH];

	if (!title)
		return NULL;

	lbl = elm_label_add(ly);
	if (!lbl) {
		return NULL;
	}

	elm_object_style_set(lbl, FBR_STYLE_LABEL);
	elm_label_ellipsis_set(lbl, EINA_TRUE);
	s = edje_object_data_get(elm_layout_edje_get(ly),
			FBR_DATA_TITLE_WIDTH);
	if (s)
		elm_label_wrap_width_set(lbl, atoi(s));
	else
		_ERR("No title width exist");

	snprintf(buf, sizeof(buf),
			"<align=left><color=#FFFFFFFF>"
			"<font=%s>"
			"<font_size=36>%s</font_size>"
			"</font></color></align>", FONT_STYLE_BOLD, _(title));
	elm_object_text_set(lbl, buf);
	elm_label_slide_speed_set(lbl, FBR_TITLE_SLIDE_SPEED);

	return lbl;
}



void CContextView::m_SetBtnDownFocus(Evas_Object *btn)
{
	ASSERT(m);

	if (!m->link_first[0] && !m->rel_first[0]) {
		elm_object_focus_next_object_set(btn, btn,
			ELM_FOCUS_DOWN);
		return;
	}

	if (m->link_first[0])
		elm_object_focus_next_object_set(btn, m->link_first[0],
		ELM_FOCUS_DOWN);
	else
		elm_object_focus_next_object_set(btn, m->rel_first[0],
		ELM_FOCUS_DOWN);
}


void CContextView::m_SetLayoutFocusOrder(Evas_Object *btn)
{
	ASSERT(btn);

	int i;
	Evas_Object *obj;

	if (m->link_first[0]) {
		elm_object_focus_next_object_set(m->link_first[0], btn,
			ELM_FOCUS_UP);
		elm_object_focus_next_object_set(m->link_first[0],
			m->rel_first[0], ELM_FOCUS_DOWN);
		elm_object_focus_next_object_set(m->link_first[1], btn,
			ELM_FOCUS_UP);
		elm_object_focus_next_object_set(m->link_first[1],
			m->rel_first[0], ELM_FOCUS_DOWN);
	}

	for (i = 0; i < TABLE_MAX_COL; i++) {
		if (!m->rel_first[i])
			continue;

		obj = m->rel_first[i];
		if (m->link_first[0])
			elm_object_focus_next_object_set(obj, m->link_first[0],
			ELM_FOCUS_UP);
		else
			elm_object_focus_next_object_set(obj, btn,
			ELM_FOCUS_UP);
	}
}


Evas_Object *CContextView::m_AddButton(Evas_Object *eoBase)
{
	ASSERT(eoBase);

	Evas_Object *btn;

	btn = elm_layout_add(eoBase);
	if (!btn) {
		_ERR("elm_layout_add failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(btn,
		EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(btn, 0.0, 0.5);

	elm_layout_file_set(btn, EDJEFILE, FBR_CONTEXT_BTN);
	elm_object_focus_allow_set(btn, EINA_TRUE);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_RIGHT);

	evas_object_smart_callback_add(btn, FBR_SIGNAL_FOCUSED,   _btn_focused, NULL);
	evas_object_smart_callback_add(btn, FBR_SIGNAL_UNFOCUSED, _btn_unfocused, NULL);

	evas_object_show(btn);
	elm_object_focus_set(btn, EINA_TRUE);

	return btn;
}


bool CContextView::m_AddTableContents(Evas_Object *table, Eina_List *list, enum content_app_type type)
{
	ASSERT(list);

	int row, col;
	Evas_Object *ly;
	Eina_List *l;
	CDummyContent::SRelativeContent *rctnt;
	void* obj;

	row = 0;
	col = 0;

	EINA_LIST_FOREACH(list, l, obj) {
		rctnt = (CDummyContent::SRelativeContent *)obj;
		ly = _add_rltv_ctnt(table, rctnt->title, rctnt->imgpath);
		if (!ly)
			continue;
		//_SET_PRIV(ly, rctnt);
		elm_table_pack(table, ly, col, row, 1, 1);
		evas_object_show(ly);

		evas_object_smart_callback_add(ly, FBR_SIGNAL_FOCUSED,       _btn_focused, NULL);
		evas_object_smart_callback_add(ly, FBR_SIGNAL_UNFOCUSED,     _btn_unfocused, NULL);

		m->pHandlerButton->Connect(ly);

		if (row == 0) {
			if (type == APP_LINKED)
				m->link_first[col] = ly;
			else
				m->rel_first[col] = ly;
		}

		if (col == 0)
			elm_object_focus_next_object_set(ly, ly, ELM_FOCUS_LEFT);
		else if (col == TABLE_MAX_COL - 1)
			elm_object_focus_next_object_set(ly, ly, ELM_FOCUS_RIGHT);

		if (type == APP_LINKED)
			m->link_last[col] = ly;
		else
			m->rel_last[col] = ly;

		col++;
		if (col >= TABLE_MAX_COL) {
			row++;
			col = 0;
		}
	}

	return true;
}


bool CContextView::m_CreateContentApps(Eina_List *list, enum content_app_type type)
{
	if (!list)
		return false;

	Evas_Object *scroller, *table;
	int r;

	scroller = _add_scroller(m->eoBase);
	if (!scroller) {
		_ERR("_add_scroller failed.");
		return false;
	}

	table = _add_table(scroller);
	if (!table) {
		_ERR("_add_table failed.");
		evas_object_del(scroller);
		return false;
	}

	r = m_AddTableContents(table, list, type);
	if (!r) {
		_ERR("m_AddTableContents failed.");
		evas_object_del(scroller);
		return false;
	}

	elm_object_content_set(scroller, table);

	if (type == APP_LINKED)
		elm_object_part_content_set(m->eoBase, FBR_PART_CTXT_LINK_AREA,
		scroller);
	else
		elm_object_part_content_set(m->eoBase, FBR_PART_CTXT_REL_AREA,
		scroller);

	return true;
}


bool CContextView::m_CreateBtnsPart(void)
{
	Evas_Object *btn, *box;
	box = _add_btnbox(m->eoBase);
	if (!box)
		return false;

	btn = m_AddButton(box);
	if (!btn) {
		evas_object_del(box);
		return false;
	}
	if (!_add_btn_info(btn, FBR_IMAGE_CTXT_OPEN, FBR_TEXT_OPEN)) {
		evas_object_del(box);
		return false;
	}

	m_SetBtnDownFocus(btn);
	m_SetLayoutFocusOrder(btn);
	elm_box_pack_end(box, btn);
	m->eoBtnOpen = btn;

	m->pHandlerButton->Connect(m->eoBtnOpen);

	elm_object_part_content_set(m->eoBase, FBR_PART_CTXT_BTNAREA, box);

	return true;
}


bool CContextView::m_CreateInfoPart(void)
{
	Evas_Object *img = NULL;
	char *title = NULL;

	char subtitle[MAX_LENGTH];
	char subtitle2[MAX_LENGTH];

	char *path = NULL;
	Eina_List *rltvlist = NULL;
	Eina_List *linkapplist = NULL;
	int type;
	Evas_Object *lbl = NULL;

	if (!_get_context_details(m->finfo, &path, &title, subtitle, subtitle2))
		return false;

	type = m->finfo->Type();
	if (!path) {
		if (type == E_GRP_PHOTO) {
			img = _get_icon(m->eoBase, m->finfo);
			if (!img)
				goto finish;
		} else {
			img = elm_image_add(m->eoBase);
			if (!img)
				goto finish;
			_set_default_thumb(img, type);
		}
	} else {
		img = elm_image_add(m->eoBase);
		if (!img)
			goto finish;
		elm_image_file_set(img, path, NULL);
		elm_image_aspect_fixed_set(img, EINA_FALSE);
	}

	elm_object_part_content_set(m->eoBase, FBR_PART_CTXT_ICON, img);

	lbl = _create_lbl(m->eoBase, title);
	if (lbl) {
		elm_object_part_content_set(m->eoBase, FBR_PART_CTXT_TITLE, lbl);
		_lbl_start_slide(lbl);
	}

	elm_object_part_text_set(m->eoBase, FBR_PART_CTXT_SUBTITLE,  subtitle);
	elm_object_part_text_set(m->eoBase, FBR_PART_CTXT_SUBTITLE2, subtitle2);

	elm_object_part_text_set(m->eoBase, FBR_PART_CTXT_LINK_TEXT, _(FBR_TEXT_LINK));
	linkapplist = CDummyContent::ListLinkApp(type);
	if (linkapplist)
		m_CreateContentApps(linkapplist, APP_LINKED);

	elm_object_part_text_set(m->eoBase, FBR_PART_CTXT_REL_TEXT, _(FBR_TEXT_REL));
	rltvlist = CDummyContent::ListRelative(type);
	if (rltvlist)
		m_CreateContentApps(rltvlist, APP_OTHER);

finish:

	return true;
}


void CContextView::sm_CbKeyPressed(void *dt, Evas *e, Evas_Object *obj, void *ei)
{
	_DBG();
	Evas_Event_Key_Down *ev;

	if (!dt || !ei)
		return;

	ev = (Evas_Event_Key_Down *)ei;

	if (!strcmp(ev->keyname, KEY_BACK))
	{
		CViewMgr::GetInstance()->PopView();
	}
	else if (!strcmp(ev->keyname, KEY_EXIT))
		elm_exit();
}

void CContextView::t_OnShow(void)
{
	evas_object_show(m->eoBase);
}

void CContextView::t_OnHide(void)
{
	evas_object_hide(m->eoBase);
}

bool CContextView::Create(void *data)
{
	ASSERT(!m);

	Evas_Object *eoBase = NULL;
	Evas_Object *eoWin = CViewMgr::GetInstance()->Window();


	_CREATE_BEGIN{
		_CHECK(m = new SContextView)
		_CHECK(eoBase = elm_layout_add(eoWin))
		_CHECK(elm_layout_file_set(eoBase, EDJEFILE, FBR_CONTEXT_VIEW))
		_COMMAND{ m->eoWin = eoWin; m->eoBase = eoBase; m->finfo = (CExtNameInfo*)data; }
		_CHECK(m_CreateInfoPart())
		_CHECK(m_CreateBtnsPart())

		_WHEN_SUCCESS{
			evas_object_event_callback_add(eoBase, EVAS_CALLBACK_KEY_DOWN, sm_CbKeyPressed, m);
		}

		_CHECK_FAIL{ /* m_CreateBtnsPart */ }
		_CHECK_FAIL{ /* m_CreateInfoPart */ }
		_CHECK_FAIL{}
		_CHECK_FAIL{ evas_object_del(eoBase);  }
		_CHECK_FAIL{ delete m; m = NULL; }
	} _CREATE_END_AND_CATCH{ return false; }

	return true;
}

void CContextView::Destroy(void)
{
	ASSERT(m);

	evas_object_del(m->eoBase);
	delete m;
	m = NULL;
}

Evas_Object* CContextView::Base(void)
{
	ASSERT(m);

	return m->eoBase;
}
